// Date: July 2015
// Author: paulbunn@email.unc.edu (Paul Bunn)
//
// Description: Reads 1st-phase (summary statistics) output of SEQMETA's
// ScoreSeq function, populating generic data structures.

#ifndef READ_SEQMETA_UTILS_H
#define READ_SEQMETA_UTILS_H

#include "PreMeta/premeta_constants.h"
#include "PreMeta/premeta_structures.h"

#include <set>
#include <string>
#include <vector>

namespace premeta {

bool ParseSeqMetaRDataGeneScores(
    const RDataList* node,
    const string& gene_name,
    map<string, vector<Position>>* gene_to_snp_positions,
    map<Position, SnpInfo>* scores);
bool ParseSeqMetaRDataGeneCovariance(
    const RDataList* node,
    const string& gene_name,
    const map<string, vector<Position>>& gene_to_snp_positions,
    map<pair<Position, Position>, double>* covariances);
bool ParseSeqMetaRDataGeneN(const RDataList* node, int* num_samples);
bool ParseSeqMetaRDataGeneMaf(
    const RDataList* node,
    const string& gene_name,
    const map<string, vector<Position>>& gene_to_snp_positions,
    map<Position, SnpInfo>* scores);
bool ParseSeqMetaRDataGeneSey(const RDataList* node, double* sey);
bool ParseSeqMetaRDataMainObjectAttributes(
    const RDataNode& main_attr, vector<string>* genes);
bool ParseSeqMetaRDataGene(
    const string& gene_name,
    const RDataNode* item, int* num_samples, double* sey,
    map<string, vector<Position>>* gene_to_snp_positions,
    map<Position, SnpInfo>* scores,
    map<pair<Position, Position>, double>* covariances);
bool ParseSeqMetaRDataTree(
    const RDataNode& root,
    int* num_samples, double* sey,
    map<string, vector<Position>>* gene_to_snp_positions,
    map<Position, SnpInfo>* scores,
    map<pair<Position, Position>, double>* covariances);

}  // namespace premeta
#endif
